@@ -7,6 +7,7 @@ use Carmel::Runner;
 use Carp ();
 use Carmel::Builder;
 use Carmel::CPANfile;
+use Carmel::Environment;
 use Carmel::Repository;
 use Carmel::Resolver;
 use Carmel::ProgressBar qw(progress);
@@ -26,7 +27,6 @@ $ENV{PERL_JSON_BACKEND} = 1
 
 use Class::Tiny {
     verbose => sub { 0 },
-    perl_arch => sub { "$Config{version}-$Config{archname}" },
 };
 
 sub parse_options {
@@ -72,21 +72,9 @@ sub run {
     return $code;
 }
 
-sub repository_base {
+sub env {
     my $self = shift;
-
-    my $home = $ENV{HOME} || $ENV{HOMEPATH};
-    Path::Tiny->new($ENV{PERL_CARMEL_REPO} || "$home/.carmel/" . $self->perl_arch);
-}
-
-sub repo {
-    my $self = shift;
-    $self->{repo} ||= $self->build_repo;
-}
-
-sub build_repo {
-    my $self = shift;
-    Carmel::Repository->new(path => $self->repository_base->child('builds'));
+    $self->{env} ||= Carmel::Environment->new;
 }
 
 sub cmd_help {
@@ -150,7 +138,7 @@ sub cmd_pin {
 sub cmd_update {
     my($self, @args) = @_;
 
-    my $snapshot = $self->snapshot
+    my $snapshot = $self->env->snapshot
       or die "Can't run carmel update without snapshot. Run `carmel install` first.\n";
 
     print "---> Checking updates...\n";
@@ -258,7 +246,7 @@ sub cmd_install {
 
     die "Usage: carmel install\n" if @args;
 
-    my $snapshot = $self->snapshot;
+    my $snapshot = $self->env->snapshot;
     if ($snapshot) {
         $self->update_dependencies($self->requirements, $snapshot);
     } else {
@@ -354,7 +342,7 @@ sub cmd_reinstall {
 
     my @modules = @args ? @args : $self->requirements->required_modules;
 
-    my $snapshot = $self->snapshot
+    my $snapshot = $self->env->snapshot
       or die "Can't run carmel reinstall without snapshot. Run `carmel install` first.\n";
 
     my $reqs = CPAN::Meta::Requirements->new;
@@ -380,9 +368,9 @@ sub builder {
     my($self, @args) = @_;
 
     Carmel::Builder->new(
-        repository_base => $self->repository_base,
-        cpanfile_path => $self->cpanfile_path,
-        collect_artifact => sub { $self->repo->import_artifact(@_) },
+        repository_base => $self->env->repository_base,
+        cpanfile_path => $self->env->cpanfile->path,
+        collect_artifact => sub { $self->env->repo->import_artifact(@_) },
         @args,
     );
 }
@@ -421,7 +409,7 @@ sub save_snapshot {
     require Carton::Snapshot;
     require Carton::Dist;
 
-    my $snapshot = Carton::Snapshot->new(path => $self->cpanfile->snapshot_path);
+    my $snapshot = Carton::Snapshot->new(path => $self->env->snapshot_path);
 
     for my $artifact (@$artifacts) {
         my $dist = Carton::Dist->new(
@@ -455,7 +443,7 @@ sub dump_bootstrap {
         %modules = (%modules, $artifact->module_files);
     }
 
-    my $prereqs = $self->cpanfile->load->prereqs->as_string_hash;
+    my $prereqs = $self->env->cpanfile->load->prereqs->as_string_hash;
     my $package = "Carmel::MySetup"; # hide from PAUSE
 
     my $file = Path::Tiny->new(".carmel/MySetup.pm");
@@ -503,7 +491,7 @@ sub cmd_exec {
 sub cmd_find {
     my($self, $module, $requirement) = @_;
 
-    my @artifacts = $self->repo->find_all($module, $requirement || '0');
+    my @artifacts = $self->env->repo->find_all($module, $requirement || '0');
     for my $artifact (@artifacts) {
         printf "%s (%s) in %s\n", $artifact->package, $artifact->version || '0', $artifact->path;
     }
@@ -552,7 +540,7 @@ sub cmd_look {
 sub cmd_diff {
     my $self = shift;
 
-    my $snapshot_path = $self->cpanfile->snapshot_path->relative;
+    my $snapshot_path = $self->env->snapshot_path->relative;
 
     # Don't check if .git exists, and let git(2) handle the error
 
@@ -593,9 +581,9 @@ sub resolver {
     my($self, @args) = @_;
 
     Carmel::Resolver->new(
-        repo     => $self->repo,
+        repo     => $self->env->repo,
         root     => $self->requirements,
-        snapshot => scalar $self->snapshot,
+        snapshot => scalar $self->env->snapshot,
         missing  => sub { $self->missing_default(@_) },
         @args,
     );
@@ -655,7 +643,7 @@ sub cmd_package {
 
     my $index = $self->build_index;
 
-    my $source_base = $self->repository_base->child('cache');
+    my $source_base = $self->env->repository_base->child('cache');
     my $target_base = Path::Tiny->new('vendor/cache');
 
     my %done;
@@ -675,7 +663,7 @@ sub cmd_package {
         } else {
             die sprintf "%s not found in %s.\n" .
               "Run `carmel install` to fix this. If that didn't resolve the issue, try removing %s\n",
-              $package->pathname, $source_base, $self->repository_base;
+              $package->pathname, $source_base, $self->env->repository_base;
         }
     }
 
@@ -719,46 +707,11 @@ sub build_index {
     $index;
 }
 
-sub cpanfile_path {
-    my $self = shift;
-    $self->{cpanfile_path} ||= Path::Tiny->new($self->locate_cpanfile)->absolute;
-}
-
-sub locate_cpanfile {
-    my $self = shift;
-
-    my $path = $ENV{PERL_CARMEL_CPANFILE};
-    if ($path) {
-        return $path;
-    }
-
-    my $current  = Path::Tiny->cwd;
-    my $previous = '';
-
-    until ($current eq '/' or $current eq $previous) {
-        my $try = $current->child('cpanfile');
-        return $try if $try->is_file;
-        ($previous, $current) = ($current, $current->parent);
-    }
-
-    return 'cpanfile'; # fallback, most certainly fails later
-}
-
-sub cpanfile {
-    my $self = shift;
-    Carmel::CPANfile->new(path => $self->cpanfile_path);
-}
-
 sub requirements {
     my $self = shift;
 
-    return $self->cpanfile->load->prereqs
+    return $self->env->cpanfile->load->prereqs
       ->merged_requirements(['runtime', 'test', 'develop'], ['requires']);
 }
 
-sub snapshot {
-    my $self = shift;
-    $self->cpanfile->load_snapshot;
-}
-
 1;
