@@ -1,10 +1,18 @@
 package Carmel::Builder;
 use strict;
 use warnings;
-use Class::Tiny qw( snapshot cpanfile cpanfile_path repository_base collect_artifact );
+use Class::Tiny qw( snapshot cpanfile cpanfile_path repository_base collect_artifact ), {
+    mirror => sub { $_[0]->build_mirror },
+};
+
+use Carmel;
 use Path::Tiny;
 use File::pushd;
 use Carmel::Lock;
+use Carton::Dist;
+use CPAN::DistnameInfo;
+use Menlo::Index::Mirror;
+use HTTP::Tinyish;
 
 sub tempdir {
     my $self = shift;
@@ -19,6 +27,17 @@ sub build_tempdir {
     Path::Tiny->tempdir(%opts);
 }
 
+sub build_mirror {
+    my $self = shift;
+
+    # FIXME: we could set the mirror option to $self->cpanfile in the caller
+    my $cpanfile = $self->cpanfile_path
+      or die "Can't locate 'cpanfile' to load module list.\n";
+
+    # one mirror for now
+    Module::CPANfile->load($cpanfile)->mirrors->[0];
+}
+
 sub install {
     my($self, @args) = @_;
 
@@ -42,12 +61,7 @@ sub install {
     local $ENV{PERL_CPANM_HOME} = $self->tempdir;
     local $ENV{PERL_CPANM_OPT};
 
-    # FIXME: we could set the mirror option to $self->cpanfile in the caller
-    my $cpanfile = $self->cpanfile_path
-      or die "Can't locate 'cpanfile' to load module list.\n";
-
-    # one mirror for now
-    my $mirror = Module::CPANfile->load($cpanfile)->mirrors->[0];
+    my $mirror = $self->mirror;
 
     my $lock = Carmel::Lock->new(path => $self->repository_base->child('run'));
     $lock->acquire;
@@ -88,14 +102,53 @@ sub search_module {
     local $ENV{PERL_CPANM_HOME} = $self->tempdir;
     local $ENV{PERL_CPANM_OPT};
 
-    my $cpanfile = $self->cpanfile_path
-      or die "Can't locate 'cpanfile' to load module list.\n";
+    my $cli = $self->cached_cli;
 
-    # one mirror for now
-    my $mirror = Module::CPANfile->load($cpanfile)->mirrors->[0];
+    if ($version && $version =~ /==|<|!/) {
+        my $dist = $cli->search_module($module, $version)
+          or return;
+
+        return Carton::Dist->new(
+            name => $dist->{distvname},
+            pathname => $dist->{pathname},
+            provides => {
+                $dist->{module} => {
+                    version => $dist->{module_version},
+                },
+            },
+            version => $dist->{version},
+        );
+    } else {
+        my $res = $self->index->search_packages({ package => $module })
+          or return;
+
+        (my $path = $res->{uri}) =~ s!^cpan:///distfile/!!;
+        my $info = CPAN::DistnameInfo->new($path);
+
+        return Carton::Dist->new(
+            name => $info->distvname,
+            pathname => $info->pathname,
+            provides => {
+                $res->{package} => {
+                    version => $res->{version},
+                },
+            },
+            version => $info->version,
+        );
+    }
+}
+
+sub cached_cli {
+    my $self = shift;
+    $self->{cli} ||= $self->build_cli();
+}
+
+sub build_cli {
+    my $self = shift;
+
+    my $mirror = $self->mirror;
 
     require Menlo::CLI::Compat;
-    require Carton::Dist;
 
     my $cli = Menlo::CLI::Compat->new;
     $cli->parse_options(
@@ -110,21 +163,24 @@ sub search_module {
     $cli->setup_home;
     $cli->init_tools;
 
-    my $dist = $cli->search_module($module, $version);
-    if ($dist) {
-        return Carton::Dist->new(
-            name => $dist->{distvname},
-            pathname => $dist->{pathname},
-            provides => {
-                $dist->{module} => {
-                    version => $dist->{module_version},
-                },
-            },
-            version => $dist->{version},
-        );
-    }
+    return $cli;
+}
 
-    return;
+sub index {
+    my $self = shift;
+    $self->{index} ||= $self->build_index;
+}
+
+sub build_index {
+    my $self = shift;
+
+    my $mirror = $self->mirror || "https://cpan.metacpan.org/";
+
+    # Use $cli->mirror to support file: URLs
+    return Menlo::Index::Mirror->new({
+        mirror  => $mirror,
+        fetcher => sub { $self->cached_cli->mirror(@_) },
+    });
 }
 
 sub rollout {
